package com.example.wc24h;

import android.app.Dialog;
import android.os.AsyncTask;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.InterfaceAddress;
import java.net.NetworkInterface;
import java.net.SocketTimeoutException;
import java.util.Enumeration;

class IPDialog extends Dialog
{
    private MainActivity                mContext = null;
    private ListView                    list;
    private SelectionListAdapter        listAdapter;
    private EditText                    ipEdit;
    private Button                      searchBtn;

    IPDialog(MainActivity a)
    {
        super(a, R.style.dialog_custom_style);

        mContext = a;
        setContentView(R.layout.ip_dialog);

//        File dir = MainActivity.getSettingsDirFile(mContext);
//        File f = new File(dir, MainActivity.getProfileFileName(i));
//        if (f.exists()) {
//            WC24hSettings tmp = new WC24hSettings();
//            tmp.loadData(dir, MainActivity.getProfileFileName(i));
//        }

        WC24hSettings settings = new WC24hSettings();
//        File f = new File(dir, MainActivity.getProfileFileName(profile));
//        if (f.exists()) {
//            settings.loadData(getSettingsDirFile(a), getProfileFileName(selectedProfile));
//        }

        // ip address edit field
        ipEdit = (EditText)findViewById(R.id.ipEdit);
        ipEdit.setText(settings.ipaddress);
        // list of found ips
        list = (ListView)findViewById(R.id.ip_list);
        listAdapter = new SelectionListAdapter(mContext);
        list.setAdapter(listAdapter);
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int position, long id) {
                Log.w("WC24h", listAdapter.getItem(position));
                listAdapter.selected = position;
                ipEdit.setText(listAdapter.getItem(position));
                listAdapter.notifyDataSetChanged();
            }
        });
        // save button
        Button btn = (Button)findViewById(R.id.save_button);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                String ipaddress = ipEdit.getText().toString().trim();

                if (ipaddress.contentEquals(""))
                {
                    InfoDialog info = new InfoDialog(mContext, R.string.ip_address_must_be_set);
                    info.show();
                }
                else
                {
                    mContext.wc24hsettings.ipaddress = ipaddress;
                    mContext.wc24hsettings.saveIPAddress(MainActivity.getSettingsDirFile(mContext), "WC24hIP.xml");
                    mContext.updateIPAddress();
                    dismiss();
                }
            }
        });
        // cancel button
        btn = (Button)findViewById(R.id.cancel_button);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                dismiss();
            }
        });
        // find button
        searchBtn = (Button)findViewById(R.id.find_btn);
        searchBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                startSearch();
            }
        });
    }

    private void startSearch() {
        if (!mContext.isNetworkAvailable(mContext)) {
            InfoDialog info = new InfoDialog(mContext, R.string.no_network_info);
            info.show();
        } else {
            searchBtn.setEnabled(false);
            listAdapter.selected = -1;
            listAdapter.clear();
            listAdapter.notifyDataSetInvalidated();
            new IpFinder(mContext).execute();
        }
    }

    @Override
    protected void onStart()
    {
        super.onStart();
        startSearch();
    }

    @Override
    protected void onStop()
    {
        super.onStop();
    }

    private class IpFinder extends AsyncTask<String, Integer, Long>
    {
        MainActivity mainActivity;
        IpFinder(MainActivity mainActivity)
        {
            this.mainActivity = mainActivity;
        }

        protected Long doInBackground(String... str)
        {
            Log.w("WC24h", "Broadcasting to find ip address...");
            byte[] sendData = "UDISCOVER_WC24h\0".getBytes();
            try {
                DatagramSocket c = new DatagramSocket();
                c.setBroadcast(true);
                c.setSoTimeout(1000);
                Enumeration interfaces = NetworkInterface.getNetworkInterfaces();
                while (interfaces.hasMoreElements()) {
                    NetworkInterface networkInterface = (NetworkInterface)interfaces.nextElement();

                    if (networkInterface.isLoopback() || !networkInterface.isUp()) {
                        continue; // Don't want to broadcast to the loopback interface
                    }

                    for (InterfaceAddress interfaceAddress : networkInterface.getInterfaceAddresses()) {
                        InetAddress broadcast = interfaceAddress.getBroadcast();
                        if (broadcast == null) {
                            continue;
                        }

                        // Send the broadcast package!
                        try {
                            DatagramPacket sendPacket = new DatagramPacket(sendData, sendData.length, broadcast, 2424);
                            c.send(sendPacket);
                        } catch (Exception e) {
                            //
                        }

                        Log.w("WC24h", "Request packet sent to " + broadcast.getHostAddress() + " via interface " + networkInterface.getDisplayName());
                    }
                }
                byte[] recvBuf = new byte[1500];
                final DatagramPacket receivePacket = new DatagramPacket(recvBuf, recvBuf.length);
                final long endTime = System.currentTimeMillis() + 10000;

                while (endTime > System.currentTimeMillis()) {
                    publishProgress((int)(endTime - System.currentTimeMillis()) / 1000);
                    try {
                        c.receive(receivePacket);

                        //We have a response
                        Log.w("WC24h", "Received broadcast response from : " + receivePacket.getAddress().getHostAddress());

                        //Check if the message is correct
                        String message = new String(receivePacket.getData()).trim();
                        Log.w("WC24h", "Received message: " + message);
                        if (message.equals("URESPONSE_WC2h")) {
                            mainActivity.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    String ip = receivePacket.getAddress().getHostAddress();
                                    listAdapter.add(ip);
                                    listAdapter.notifyDataSetInvalidated();
                                    if (ipEdit.getText().toString().trim().length() == 0) {
                                        ipEdit.setText(ip);
                                    }
                                }
                            });
                        }
                    } catch (SocketTimeoutException e) {
                        Log.w("WC24h", "Broadcast timeout...");
                    }
                }
                Log.w("WC24h", "Broadcast end");

                //Close the port!
                c.close();
            } catch (IOException ex) {
                Log.e("WC24h", "", ex);
            }

            return null;
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            Toast.makeText(mainActivity.getBaseContext(), R.string.searching_for_the_wordclock, Toast.LENGTH_SHORT).show();
        }

        protected void onProgressUpdate(Integer... progress)
        {
            super.onProgressUpdate(progress);
            String str = mainActivity.getString(R.string.btn_find_ip_s, progress[0]);
            searchBtn.setText(str);
        }

        protected void onPostExecute(Long result)
        {
            super.onPostExecute(result);
            searchBtn.setText(R.string.btn_find_ip);
            searchBtn.setEnabled(true);
        }
    }
}